
/**
 * Classe Dicionario 
 * 
 * Implementa uma correspondncia entre um TERMO (chave)
 * e um ARRAYLIST contendo os respectivos SIGNIFICADOS.
 * 
 * NOTA: Strings no necessitam de ser clonadas !!
 * 
 * @author F. Mrio Martins
 * @version 1/2005
 */
import java.util.*;
import java.io.*;
public class Dicionario {
	
	// Varivel de Instncia
	
	private TreeMap<String, ArrayList<String>> dicio; 

	/** Construtor por omisso */
	public Dicionario() { dicio = new TreeMap<String, ArrayList<String>>(); }
	
	/** Construtor de cpia */
	public Dicionario(Dicionario dic) {
	    dicio = new TreeMap<String, ArrayList<String>>();
	    for(String termo : dic.daTermos()) {
	       ArrayList<String> signifs = 
	             new ArrayList<String>(dic.significados(termo));
	       dicio.put(termo, signifs);
	    }
	}

	/** Devolve todos os TERMOS do dicionario (chaves) */
	public TreeSet<String> daTermos() { 
	    TreeSet<String> termos = new TreeSet<String>(dicio.keySet());
	    return termos; 
	}
		
	/** D o nmero total de termos do dicionario (chaves) */
	public int numTermos() { return dicio.keySet().size(); }
	
	/** Determina se existe um dado termo */
	public boolean existeTermo(String termo) { 
	    return dicio.containsKey(termo); 
	}
	
	/**
	 * Devolve o Set dos significados de um dado termo.
	 * O mtodo admite que o termo existe.
	 */
	public TreeSet<String> significados(String termo) {
	    TreeSet<String> aux = new TreeSet<String>();
	    for(String sig : dicio.get(termo)) aux.add(sig);
	    return aux;
    }
   
   /**
	 * Devolve o nmero total de significados existentes no 
	 * dicionrio.
	 */
	public int numSignificados()  {	    
	    int total = 0;
	    for(ArrayList<String> signifs : dicio.values()) 
	       for(String sign : signifs) total++;
	   return total;
    }
    
     /**
     * Mtodo que insere um novo significado para um dado termo
     * j existente no dicionario
     */
    public void insereSignificado(String termo, String signif) {
       ArrayList<String> signifs = dicio.get(termo);
       signifs.add(signif); 
    }
    
    /**
     * Mtodo que insere um novo termo e os respectivos significados.
     * O mtodo admite que o termo , de facto, novo.
     */
    public void insereTermo(String termo, ArrayList<String> signifs) {
       ArrayList<String> sigs = new ArrayList<String>(signifs);  // usa construtor para copiar
       dicio.put(termo, sigs);
    }
       
    /**
     * Mtodo que recebe um Dicionario como parmetro e devolve um
     * Set com os termos comuns entre o parmetro e o receptor.
     */ 
    public Set<String> comuns(Dicionario dicPar) {
        TreeSet<String> aux = dicPar.daTermos();
        aux.retainAll(dicio.keySet());
        return aux;
    }
    
    /**
     * Mtodo que devolve o termo com mais significados
     */ 
    public String termoMaisSign() {
        String termoMais = ""; 
        int maisSign = 0; int numSign = 0;
        for(String termo : dicio.keySet()) {
            numSign = dicio.get(termo).size();
            if(  numSign > maisSign ) {
                 termoMais = termo; maisSign = numSign;
            }
        }
        return termoMais;
    }
    
       
    /**
     * Mtodo que devolve um ArrayList dos significados todos que tambm 
     * so termos (chaves) no dicionrio
     */
    public Set<String> signiTermos() {
        HashSet<String> saoTermos = new HashSet<String>();
        String sig = "";
          for(ArrayList<String> signifs : dicio.values()) 
            for(String signif : signifs) 
	            if(dicio.containsKey(signif)) saoTermos.add(sig);
	    return saoTermos;
    }
        
     /**
     * Mtodo que dado um significado devolve um Set dos termos
     * de que tal palavra  significado.
     */
    public Set<String> e_Significado_De(String signif) {
        ArrayList<String> signifs = new ArrayList<String>();
        HashSet<String> chaves = new HashSet<String>();
        for(String termo : dicio.keySet()) {
	          signifs = dicio.get(termo);
	          if(signifs.contains(signif)) chaves.add(termo);
	    }	  
	   return chaves;
    }
    
    /**
     * Mtodo que junta o Dicionrio parmetro ao receptor.
     * Se um termo  novo adiciona o novo par termo-significados. 
     * Se o termo j existe, adiciona os novos significados para
     * tal termo.
     */
    public void juntaDicionario(Dicionario dicPar) {
        ArrayList<String> sigsPar = null;
        ArrayList<String> signifs = new ArrayList<String>();
        for(String termo : dicPar.daTermos()) {
            sigsPar = new ArrayList<String>(dicPar.significados(termo));
            if(dicio.containsKey(termo)) {
                 signifs = dicio.get(termo); 
                 signifs.addAll(new HashSet<String>(sigsPar));
              }     
            else 
               dicio.put(termo, sigsPar);                
        }         	   
    }

    /**
     * Mtodo toString().
     * Para cada termo apresenta os respectivos significados.
     * linha a linha.
     */
    public String toString() {
        StringBuilder strb = new StringBuilder();        
        strb.append("\t\t------------ DICIONARIO ----------------\n\n");
        for(String termo : dicio.keySet()) {
	        strb.append("--- SIGNIFICADOS DE: " + termo + "\n");
	        for(String sig : dicio.get(termo)) {
	            strb.append("\t" + sig + "\n"); 	            
	        }
	        strb.append("--------------------\n");
	    }
	 return strb.toString();
	}
	
	/** equals => ter o mesmo conjunto de chaves ... */
	public boolean equals(Object obj) {
	    if(this == obj) return true;
	    if( (obj == null) || (obj.getClass() != this.getClass()) ) return false;
	    //
	    Dicionario dicPar = (Dicionario) obj;
	    TreeSet<String> chavesPar = dicPar.daTermos();
	    TreeSet<String> chaves = this.daTermos();
	    return chavesPar.containsAll(chaves);
    }
	  
   /** clone */
   public Dicionario clone() { return new Dicionario(this); }

}
